'''Program to use cube_combine.py to combine a series of images'''
import numpy
import pyfits
import scipy
import ndimage
import numpy.ma as ma
import cube_combine as cc
import time
import pylab as pl # RH
import sys         # RH

def image_combine(image_list, offsets=[], bpm_threshold=0, bpm_list=[],
                  (offset, integer)=(True, False),  # RH
                  background=True): # RH
    '''
    This function combines the given images using the information given

    Input:
        image_list = List of the images to be combined, given as numpy arrays.

        offsets = Optional. Tuple of lists of the relative offset of each image.
                  The 'zeroth' image has an offset of (0,0).
                  These must be given in (x,y) form.

        bpm_list = Optional. List of the bad pixel mask arrays. These must be of
                   the same shape as the images.
                   By default, all the pixels are assumed to be good.

        bpm_threshold = Optional. The threshold value of the bpm array. If the
                        absolute value of an entry is above this threshold, the
                        corresponding pixel in the image is masked as bad. This
                        is set to 0 by default.

        offset,integer = Optional. Use to determine if the images need to be
                         offset relative to each other. If so, the offset must
                         be True. If integer is true, then a quicker offset is
                         performed. Useful for cases where the offsets are
                         actually integers in pixel units or if a rough image is
                         desired quickly. Otherwise, the images are interpolated
                         during the offset procedure.

        background  = Perform pairwise background subtraction (1-2, 3-4, 5-6 etc)

    Output:
        Returns one final image, which, in general, will be larger than any of
        the given images due to the offsetting. The images are averaged
        appropriately, taking into account any bad pixels. Thus, the output is
        normalized for each case. Also returns its corresponding quality array,
        the entries of which are the number of good pixels each entry in the
        image represents.
        
        '''
    
    # Perform pairwise background subtraction if necessary.
    
    if background:
        bkgrnd = []
        number = len(image_list)
        # Straightforward if an even number of images given.
        if number%2 == 0:
            for k in range(0,number,2):
                bk = cc.background_remove(image_list[k], image_list[k+1])
                bkgrnd.append(bk[0])
                bkgrnd.append(bk[1])

        else:
            # Repeat as before, but leaving the last image out.
            for k in range(0,number-1, 2):
                bk = cc.background_remove(image_list[k], image_list[k+1])
                bkgrnd.append(bk[0])
                bkgrnd.append(bk[1])
            # For the last image, find the image that has the largest offset
            # relative to it.
            differences = []
            for i, offset in enumerate(offsets[0]):
                difference = numpy.absolute(offsets[0][-1]-offset)
                differences.append(difference)
            furthest_index = differences.index(max(differences))
            # Now do background subtraction between these two images
            bklast = cc.background_remove(image_list[-1],
                                          image_list[furthest_index])
            bkgrnd.append(bklast[0])
    else:
        bkgrnd = image_list

    if offset:
        # Offset the images with their appropriate offset.
        offset_images = cc.offset(bkgrnd, offsets, integer)
        # Offset the BPM arrays if given.
        if bpm_list:
            offset_bpm = cc.offset(bpm_list, offsets, integer)
        else:
            offset_bpm = []
        combined_image = cc.precise_combine(offset_images, offset_bpm,
                                                bpm_threshold)
        final_image = combined_image[0]
        bpm_information = combined_image[1]
        return final_image, bpm_information
        
    else:
        combined_image = cc.precise_combine(bkgrnd, bpm_list, bpm_threshold)
        final_image = combined_image[0]
        bpm_information = combined_image[1]
        return final_image, bpm_information
        

            
if __name__ == '__main__':
    
    """
    RH edit: if run from the shell, make it a quickfire script using args as
    input
    """
    start_time = time.time()

        
    if(len(sys.argv) == 3):
        files = pl.loadtxt(sys.argv[1],comments="#",dtype="S25")
        offx,offy = pl.loadtxt(sys.argv[2],comments="#", unpack=True)
        offsets=(offx,offy)
        bpms=[]
        bpmthresh=0.0
    elif (len(sys.argv)==5):
        files = pl.loadtxt(sys.argv[1],comments="#",dtype="S25")
        offx,offy = pl.loadtxt(sys.argv[2],comments="#", unpack=True)
        offsets=(offx,offy)
        bpms  = pl.loadtxt(sys.argv[3],comments="#",dtype="S25")
        bpmthresh = numpy.float(sys.argv[4])
    else:
        print "Wrong input. Quitting program."
        exit

    # open sci images
    imagelist=[]
    headerlist = []
    for file in files:
        imagelist.append((pyfits.open(file))[0].data)
        headerlist.append((pyfits.open(file))[0].header)

    # open bpms - 
    bpmlist=[]
    for file in bpms:
        bpmlist.append((pyfits.open(file))[0].data)
        
    # Create a new FITS file for the combined image.
    aim = image_combine(imagelist, offsets, bpmthresh, bpmlist, background=False)
    # Use the leftmost image's header
    min_x_offset = min(offsets[0])
    locations = numpy.where(offsets[0]==min_x_offset)
    left_location = locations[0][0]
    y_offset = offsets[1][left_location]
    header_template = headerlist[left_location]
    # Update the reference pixel keywords. Only the y reference must be changed.
    min_y_offset = min(offsets[1])
    if y_offset == min_y_offset:
        y_ref = 0
    elif y_offset >= 0:
        y_ref = abs(min_y_offset)+y_offset
    else:
        y_ref = abs(min_y_offset)-abs(y_offset)
    y_pos = y_ref + header_template['CRPIX2']
    header_template['CRPIX2'] = y_pos
    # Write the complete FITS file with data and header.
    hdu = pyfits.PrimaryHDU(aim[0], header_template)
    hdulist = pyfits.HDUList([hdu])
    hdulist.writeto("combined.fits")
    print 'Run time', time.time()-start_time, 'seconds'
    


